'use strict';

/**
 * ncc上下文信息
 * @author xg
 */
var _ = require('lodash');
var sa = require('superagent');
var Cookies = require('js-cookie');
var IsIE = require('./IsIE');
var noCache = require('./NoCache');
var USER_CONTEXT_KEY = "STOREFORINFODATA"; // localStorage中存储用户和集团信息的key(该键值与ncc保持一致)
var BUSINESS_CONTEXT_KEY = "NCC_BUSINESS_INFO"; // localStorage中存储当前登录用户业务操作相关信息

// 判断当前运行环境是否生产环境
function isProdEnv() {
    console.log("process.env.YYLIB_ENV", process.env.YYLIB_ENV);
    return process.env.YYLIB_ENV === "prod";
}

var NccContext = {
    /**
     * 初始化ncc上下文信息
     * @param options 参数配置{
     *  url:获取上下文的URL
     *  success:初始化上下文成功时的回调
     *  error:初始化上下文失败时的回调
     *  cookieOnly:只获取cookie,默认false
     * }
     */
    init: function init(options) {
        var _this = this;

        var _ref = options || {},
            url = _ref.url,
            success = _ref.success,
            _error = _ref.error,
            cookieOnly = _ref.cookieOnly;

        if (!url) {
            console.error("初始化ncc上下文失败: url为空");
            return;
        }
        var params = {
            "busiParamJson": "{}",
            "sysParamJson": {}
        };
        if (cookieOnly === true) {
            // 只获取cookie, 设计器和组件库使用
            this.request({
                url: url,
                method: 'POST',
                send: params,
                cookies: true,
                noToken: true,
                success: function success(result) {
                    if (result && result.success) {
                        // 写入Cookie
                        var cookies = result.data || {};
                        for (var key in cookies) {
                            if (cookies.hasOwnProperty(key)) {
                                Cookies.set(key, cookies['' + key]);
                            }
                        }
                    }
                },
                error: function error() {
                    if (_error && _.isFunction(_error)) {
                        _error();
                    } else {
                        console.error("初始化ncc上下文失败");
                    }
                }
            });
        } else if (!isProdEnv()) {
            this.request({
                url: url,
                method: 'POST',
                send: params,
                cookies: true,
                noToken: true,
                success: function success(result) {
                    if (result && result.success) {
                        // 写入Cookie
                        var cookies = result.data || {};
                        for (var key in cookies) {
                            if (cookies.hasOwnProperty(key)) {
                                Cookies.set(key, cookies['' + key]);
                            }
                        }
                        _this.queryUserInfo(options);
                    }
                },
                error: function error() {
                    if (_error && _.isFunction(_error)) {
                        _error();
                    } else {
                        console.error("初始化ncc上下文失败");
                    }
                }
            });
        } else {
            this.queryBusinessInfo(options);
        }
    },

    // 获取登录用户信息
    queryUserInfo: function queryUserInfo(options) {
        var _this2 = this;

        var _ref2 = options || {},
            url = _ref2.url;

        var a = document.createElement('a');
        a.href = url;
        var baseHost = a.protocol + '//' + a.host;
        var queryUrl = baseHost + "/nccloud/platform/pub/mergerequest.do";
        var params = {
            "busiParamJson": JSON.stringify([{
                rqUrl: "/platform/appregister/querypersonsettings.do",
                rqCode: "querypersonsettings"
            }, { rqUrl: "/platform/userimage/queryuserimage.do", rqCode: "queryuserimage" }]),
            "sysParamJson": { appcode: "10228888", pagecs: Cookies.get("cookiets") }
        };
        this.request({
            url: queryUrl,
            method: 'POST',
            send: params,
            cookies: true,
            noToken: true,
            success: function success(result) {
                if (result && result.success) {
                    console.info("获取登录用户信息成功");
                    // 写入LocalStorage
                    var data = result.data ? result.data.querypersonsettings : null;
                    if (data) {
                        window.localStorage.setItem(USER_CONTEXT_KEY, JSON.stringify(data));
                    } else {
                        window.localStorage.removeItem(USER_CONTEXT_KEY);
                    }
                    _this2.queryBusinessInfo(options);
                }
            },
            error: function error() {
                console.error("获取登录用户信息失败");
                window.localStorage.removeItem(USER_CONTEXT_KEY);
            }
        });
    },

    // 查询业务需要的相关信息(汇率, 业务单元，部门，人员，项目，精度参数，含税/无税优先)
    queryBusinessInfo: function queryBusinessInfo(options) {
        var _ref3 = options || {},
            url = _ref3.url,
            _success = _ref3.success,
            _error2 = _ref3.error;

        var a = document.createElement('a');
        a.href = url;
        var baseHost = a.protocol + '//' + a.host;
        var queryUrl = baseHost + "/nccloud/jzbase/jzweb/pubMethodService.do";
        var params = {
            "busiParamJson": JSON.stringify({ method: "queryJzParam" }),
            "sysParamJson": { pagecs: Cookies.get("cookiets") }
        };
        this.request({
            url: queryUrl,
            method: 'POST',
            send: params,
            cookies: true,
            noToken: true,
            success: function success(result) {
                if (result && result.success) {
                    console.info("获取登录用户业务信息成功");
                    // 写入LocalStorage
                    if (result.data) {
                        window.localStorage.setItem(BUSINESS_CONTEXT_KEY, JSON.stringify(result.data));
                    } else {
                        window.localStorage.removeItem(BUSINESS_CONTEXT_KEY);
                    }
                    _success && _success();
                }
            },
            error: function error() {
                console.error("获取登录用户业务信息失败");
                window.localStorage.removeItem(BUSINESS_CONTEXT_KEY);
                _error2 && _error2();
            }
        });
    },

    // 获取当前登录用户ID
    getUserId: function getUserId() {
        return this.getUserContext()["userId"];
    },

    // 获取当前登录用户编号
    getUserCode: function getUserCode() {
        return this.getUserContext()["userCode"];
    },

    // 获取当前登录用户名称
    getUserName: function getUserName() {
        return this.getUserContext()["userName"];
    },

    // 获取登录用户默认业务单元
    getOrgInfo: function getOrgInfo() {
        return this.getBusinessInfo()["pk_org"];
    },

    // 获取登录用户默认业务单元ID
    getOrgId: function getOrgId() {
        var orgInfo = this.getOrgInfo();
        return orgInfo && orgInfo["id"] ? orgInfo["id"] : null;
    },

    // 获取登录用户默认业务单元名称
    getOrgName: function getOrgName() {
        var orgInfo = this.getOrgInfo();
        return orgInfo && orgInfo["name"] ? orgInfo["name"] : null;
    },

    // 获取登录用户默认项目信息
    getProjectInfo: function getProjectInfo() {
        return this.getBusinessInfo()["pk_project"];
    },

    // 获取登录用户默认项目ID
    getProjectId: function getProjectId() {
        var projectInfo = this.getProjectInfo();
        return projectInfo && projectInfo["id"] ? projectInfo["id"] : null;
    },

    // 获取登录用户默认项目名称
    getProjectName: function getProjectName() {
        var projectInfo = this.getProjectInfo();
        return projectInfo && projectInfo["name"] ? projectInfo["name"] : null;
    },

    // 获取登录用户关联的人员信息
    getPsndocInfo: function getPsndocInfo() {
        return this.getBusinessInfo()["pk_psndoc"];
    },

    // 获取登录用户关联的人员ID
    getPsndocId: function getPsndocId() {
        var psndocInfo = this.getPsndocInfo();
        return psndocInfo && psndocInfo["id"] ? psndocInfo["id"] : null;
    },

    // // 获取登录用户关联的人员名称
    getPsndocName: function getPsndocName() {
        var psndocInfo = this.getPsndocInfo();
        return psndocInfo && psndocInfo["name"] ? psndocInfo["name"] : null;
    },

    // 获取登录用户所属部门信息
    getDeptInfo: function getDeptInfo() {
        return this.getBusinessInfo()["pk_dept"];
    },

    // 获取登录用户所属部门ID
    getDeptId: function getDeptId() {
        var deptInfo = this.getDeptInfo();
        return deptInfo && deptInfo["id"] ? deptInfo["id"] : null;
    },
    // 获取登录用户所属部门名称
    getDeptName: function getDeptName() {
        var deptInfo = this.getDeptInfo();
        return deptInfo && deptInfo["name"] ? deptInfo["name"] : null;
    },

    // 获取本币
    getCurrencyId: function getCurrencyId() {
        return this.getBusinessInfo()["ccurrencyid"];
    },

    // 获取原币
    getOrgCurrencyId: function getOrgCurrencyId() {
        return this.getBusinessInfo()["corigcurrencyid"];
    },

    // 获取全局汇率
    getGlobalExchangeRate: function getGlobalExchangeRate() {
        return this.getBusinessInfo()["nglobalexchgrate"];
    },

    // 获取集团汇率
    getGroupExchangeRate: function getGroupExchangeRate() {
        return this.getBusinessInfo()["ngroupexchgrate"];
    },

    // 获取组织汇率
    getOrgExchangeRate: function getOrgExchangeRate() {
        return this.getBusinessInfo()["nexchangerate"];
    },

    // 获取精度参数设置
    getScales: function getScales() {
        var scales = this.getBusinessInfo()["MnyBizParam"] || {};
        return {
            money: parseInt(scales["JZ000"]), // 金额精度
            num: parseInt(scales["JZ001"]), // 数量精度
            price: parseInt(scales["JZ002"]), // 单价精度
            rate: parseInt(scales["JZ004"]) // 税率精度
        };
    },

    // 获取金额互算方式（0 含税优先 | 1 无税优先）
    getMnyCalType: function getMnyCalType() {
        var scales = this.getBusinessInfo()["MnyBizParam"] || {};
        var type = scales["JZ003"];
        if (type === "含税优先") {
            return 0;
        } else {
            return 1;
        }
    },

    // 获取集团ID
    getGroupId: function getGroupId() {
        return this.getCurrentGroup()["pk_group"];
    },

    // 获取集团编号
    getGroupCode: function getGroupCode() {
        return this.getCurrentGroup()["groupCode"];
    },

    // 获取集团名称
    getGroupName: function getGroupName() {
        return this.getCurrentGroup()["groupName"];
    },

    // 获取当前选择的集团
    getCurrentGroup: function getCurrentGroup() {
        var groupVOs = this.getUserContext()["groupVOs"] || [];
        groupVOs = groupVOs.filter(function (groupVO) {
            return groupVO["is_selected"] === true;
        });
        return groupVOs && groupVOs.length === 1 ? groupVOs[0] : {};
    },

    // 获取用户信息
    getUserContext: function getUserContext() {
        if (!window.localStorage) {
            return {};
        }
        return window.localStorage[USER_CONTEXT_KEY] ? JSON.parse(window.localStorage[USER_CONTEXT_KEY]) : {};
    },

    // 获取用户业务信息上下文
    getBusinessInfo: function getBusinessInfo() {
        if (!window.localStorage) {
            return {};
        }
        return window.localStorage[BUSINESS_CONTEXT_KEY] ? JSON.parse(window.localStorage[BUSINESS_CONTEXT_KEY]) : {};
    },

    _defaultHeader: {},

    //设置全局缺省配置
    _defaultOptions: {
        method: 'GET', // 默认请求方法
        type: 'json', // 请求体发送方式 application/json|json|png
        url: '', // 请求url
        header: null, //{object}设置当前请求的header信息
        cookies: false, //是否携带cookies信息
        noToken: false, // 是否在请求中禁用token信息
        //reloginIfTokenExpired: true, // 是否token失效自动跳转到登录界面
        send: '', //POST请求参数，即参数不会拼接在URL后面。
        query: '', //GET请求参数，即参数拼接在URL后面。
        // type : '', // 默认content-type，支持text/html|application/x-www-form-urlencoded|application/json|multipart/form-data
        // accept : '', // 默认接受类型application/json|json|text|text/plain
        success: null, // 请求成功的回调函数
        error: null, // 请求失败的回调函数，包括http客户端错误、服务端错误、以及接口调用成功但存在业务错误
        complete: null // 请求完成的回调函数(不管成功还是失败)
    },

    request: function request(options) {
        if (options.success === undefined) delete options.success;
        if (options.error === undefined) delete options.error;
        if (options.complete === undefined) delete options.complete;
        if (options.cookies === undefined) delete options.cookies;
        var _defaults = _.assign({}, this._defaultOptions, options);
        // 禁用token
        if (options.query && options.query.noToken !== undefined) {
            _defaults.noToken = options.query.noToken;
        }
        // 发送ajax请求
        var contentType = 'application/json;charset=UTF-8';
        if (_defaults.type === 'form') {
            contentType = 'application/x-www-form-urlencoded;charset=UTF-8';
        }
        var finalNoCache = IsIE === -1 ? noCache : noCache.withQueryStrings;
        var req = sa(_defaults.method, _defaults.url).use(finalNoCache).set('Content-Type', contentType);
        //设置全局的header信息
        for (var header in this._defaultHeader) {
            if (this._defaultHeader[header] != undefined) {
                req.set(header, this._defaultHeader[header]);
            }
        }
        //设置当前请求的header信息
        if (_defaults.header && _.isPlainObject(_defaults.header)) {
            for (var headerKey in _defaults.header) {
                if (_defaults.header[headerKey] != undefined) {
                    req.set(headerKey, _defaults.header[headerKey]);
                }
            }
        }
        //设置请求携带cookie
        if (_defaults.cookies == true) {
            req.withCredentials();
        }
        req.query(_defaults.query).send(_defaults.send).end(function (err, res) {
            if (err || res && res.badRequest) {
                if (typeof _defaults.error == 'function') {
                    //如果有外部的错误异常处理则使用外部的
                    _defaults.error(res);
                } else {
                    //如果没有则使用内部默认的异常处理
                    // alert('请求异常[状态码:'+err.status+']\n'+err.message);
                    console.error('请求异常[状态码:' + err.status + ']', err);
                }
            }
            if (res && res.ok) {
                if (res.headers && res.headers['icop-content-type']) {
                    if (typeof _defaults.error == 'function') {
                        //如果有外部的错误异常处理则使用外部的
                        _defaults.error(res);
                    } else {
                        //如果没有则使用内部默认的异常处理
                        //捕获服务端包装的异常消息
                        alert('异常错误\n' + (res.text && res.text.length > 0 ? res.text : res.body));
                        console.error('异常错误', res);
                    }
                    return;
                } else {
                    if (typeof _defaults.success == 'function') {
                        if (_defaults.accept == 'text') {
                            _defaults.success(res.text);
                        } else {
                            _defaults.success(res.body);
                        }
                    }
                }
            }
            if (_defaults.complete) {
                _defaults.complete(res);
            }
        });
    }
};
module.exports = NccContext;